import numpy,sys,genutil


def afunc(func,vls,nms,prepend='',nxtrm=2,verbose=True):        
    vals=(vls,)
    out=func(*vals)
    if verbose: print '\t\t%s%s: %g' % (prepend,func.__name__,out)
    args = numpy.ma.argsort(vls)
    if verbose:
        print '\t\t\tExtremes: ',
        for i in range(nxtrm):
            if len(nms)>i:
                print '%s, %g; ' % (nms[args[i]],vls[args[i]]),
        for i in range(nxtrm):
            if len(nms)>i:
                print '%s, %g; ' % (nms[args[len(nms)-i-1]],vls[args[len(nms)-i-1]]),
        print
    vls = numpy.ma.sort(vls)
    done = False
    while nxtrm>0 and done is False:
        if len(nms)>nxtrm*2:
            vls=vls[nxtrm:-nxtrm]
            vals=(vls,)
            out = func(*vals)
            done = True
            if verbose: print '\t\t\t%s%s (no extremes): %g' % (prepend,func.__name__,out)
        else:
            nxtrm-=1
    return out


def process_file(file,doavg=False,results={},verbose=True):
    vars={}
    f=open(file)
    offset=0
    if doavg:
        offset=1

    for l in f.xreadlines():
        sp=l.split()
        if len(sp)==1:
            continue
        var=sp[1]
        vr=vars.get(var,{})
        modl = sp[0].split("/")[10]
        if sp[2]=='N/A':
            lev='N/A'
        else:
            lev=str(int(float(sp[2])*100)/100)
        levd=vr.get(lev,{})
        mins = levd.get("mins",[])
        maxs = levd.get("maxs",[])
        avgs = levd.get("avgs",[])
        amins = levd.get("amins",[])
        amaxs = levd.get("amaxs",[])
        aavgs = levd.get("aavgs",[])
        modnm = levd.get("names",[])
        try:
            mn = float(sp[3])
        except:
            mn = 1.e20
        try:
            mx = float(sp[4])
        except:
            mx = 1.e20
        if doavg:
            try:
                avg = float(sp[5])
            except:
                avg = 1.e20
        else:
            avg=1.e20
        try:
            amn = float(sp[5+offset])
        except:
            amn = 1.e20
        try:
            amx = float(sp[6+offset])
        except:
            amx = 1.e20
        if doavg:
            try:
                aavg = float(sp[8])
            except:
                aavg = 1.e20
        else:
            aavg=1.e20

        if modl in modnm:
            mmn = mins.pop(modnm.index(modl))
            mmx = maxs.pop(modnm.index(modl))
            mavg = avgs.pop(modnm.index(modl))
            mamn = amins.pop(modnm.index(modl))
            mamx = amaxs.pop(modnm.index(modl))
            maavg = aavgs.pop(modnm.index(modl))
            mnm = modnm.pop(modnm.index(modl))
            if mn!=1.e20 :  mn = min(mmn,mn)
            if mx!=1.e20 :  mx = max(mmx,mx)
            if avg!=1.e20 :  avg = (avg+mavg)/2.
            if amn!=1.e20 : amn = min(mamn,amn)
            if amx!=1.e20 : amx = max(mamx,amx)
            if aavg!=1.e20 :  aavg = (aavg+maavg)/2.

        mins.append(mn)
        maxs.append(mx)
        avgs.append(avg)
        amins.append(amn)
        amaxs.append(amx)
        aavgs.append(aavg)
        modnm.append(modl)

        levd['mins']=mins
        levd['maxs']=maxs
        levd['avgs']=avgs
        levd['amins']=amins
        levd['amaxs']=amaxs
        levd['names']=modnm
        levd['aavgs']=aavgs
        vr[lev]=levd
        vars[var]=vr

    vrs=vars.keys()
    vrs.sort()
    for vr in vrs:
        if verbose: print vr
        if vr in results.keys():
            print 'Var:',vr,'already exists skipping'
            continue
        V={}
        var=vars[vr]
        #print 'vaR:',var
        levs = var.keys()
        levs.sort()
        for l in levs:
            L=var[l]
            mins=numpy.ma.masked_greater(L['mins'],1.e20)
            maxs=numpy.ma.masked_greater(L['maxs'],1.e20)
            avgs=numpy.ma.masked_greater(L['avgs'],1.e20)
            amins=numpy.ma.masked_greater(L['amins'],1.e20)
            amaxs=numpy.ma.masked_greater(L['amaxs'],1.e20)
            aavgs=numpy.ma.masked_greater(L['aavgs'],1.e20)
            nms = L['names']
            if verbose: print '\tLevel:',l,len(nms),'models',len(mins),len(avgs)
            
            Mn  = afunc(numpy.ma.min,mins,nms,verbose=verbose)
            Mna = afunc(numpy.ma.average,mins,nms,'Mins ',verbose=verbose)
            Mns = afunc(genutil.statistics.std,mins,nms,'Mins ',verbose=verbose)

            Mx  = afunc(numpy.ma.max,maxs,nms,verbose=verbose)
            Mxa = afunc(numpy.ma.average,maxs,nms,'Maxs ',verbose=verbose)
            Mxs = afunc(genutil.statistics.std,maxs,nms,'Maxs ',verbose=verbose)

            if doavg:
                Am = afunc(numpy.ma.min,avgs,nms,'Averages ',verbose=verbose)
                AM = afunc(numpy.ma.max,avgs,nms,'Averages ',verbose=verbose)
                Aa = afunc(numpy.ma.average,avgs,nms,'Averages ',verbose=verbose)
                As = afunc(genutil.statistics.std,avgs,nms,'Averages ',verbose=verbose)
                
            AMn  = afunc(numpy.ma.min,amins,nms,'Absolute ',verbose=verbose)
            AMna = afunc(numpy.ma.average,amins,nms,'Abs. Mins ',verbose=verbose)
            AMns = afunc(genutil.statistics.std,amins,nms,'Abs. Mins ',verbose=verbose)
            
            AMx  = afunc(numpy.ma.max,amaxs,nms,'Absolute ',verbose=verbose)
            AMxa = afunc(numpy.ma.average,amaxs,nms,'Abs. Maxs ',verbose=verbose)
            AMxs = afunc(genutil.statistics.std,amaxs,nms,'Abs. Maxs ',verbose=verbose)
            
            if doavg:
                AAm = afunc(numpy.ma.min,aavgs,nms,'Abs. Averages ',verbose=verbose)
                AAM = afunc(numpy.ma.max,aavgs,nms,'Abs. Averages ',verbose=verbose)
                AAa = afunc(numpy.ma.average,aavgs,nms,'Abs. Averages ',verbose=verbose)
                AAs = afunc(genutil.statistics.std,aavgs,nms,'Abs. Averages ',verbose=verbose)

            Lev={'Min':{'min':Mn,'avg':Mna,'std':Mns},
                 'Max':{'max':Mx,'avg':Mxa,'std':Mxs},
                 'AMin':{'min':AMn,'avg':AMna,'std':AMns},
                 'AMax':{'max':AMx,'avg':AMxa,'std':AMxs}
                 }
            if doavg:
                Lev['Avg']={'max':AM,'min':Am,'avg':Aa,'std':As}
                Lev['AAvg']={'max':AAM,'min':AAm,'avg':AAa,'std':AAs}
            V[l]=Lev
        results[vr]=V
    f.close()
    return results


if __name__=="__main__":
    import pickle

    f=open("Tables_csv/minmax.pickled",'w')

    myvars={}
    for fnm in sys.argv[1:]:
        print 'Processing:',fnm
        myvars = process_file(fnm,doavg=True,results=myvars,verbose=False)
        print len(myvars)

    pickle.dump(myvars,f)
    f.close()
    #print myvars



#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Mon Jul 18 13:49:08 2016

This script generates all json files required to run obs4MIPs data creation and
then calls cmor3 to generate two example files.

The file is split into 3 sections:
    1 - creates controlled vocabulary (CV) json inputs for CMOR3
    2 - creates an Omon (ocean monthly) json input for generating a 'tos'
        variable from a CF-compliant input
    3 - creates an fx (fixed field) json input for generating an 'areacello'
        variable from a CF-compliant input

PJD 20 Jul 2016     - Removed source_id
PJD 20 Jul 2016     - Further tweaks to enhance readability
                    - TODO:

@author: durack1
"""

execfile('read_json_fcns.py')


#%% Import statements
import cmor,gc,json,os,ssl,sys,urllib2
import cdms2 as cdm
import numpy as np

#%% Set local path
homePath = os.path.join('/','/'.join(os.path.realpath(__file__).split('/')[0:-1])) ; # Extract path from executing file
#homePath = '/export/durack1/git/obs4MIPs-cmor-tables/' ; # Hard code path
#homePath = '/sync/git/obs4MIPs-cmor-tables/demo' ; # Hard code path
os.chdir(homePath)


#%% SECTION 2 - Integrate Omon into master file - create obs4MIPs_Omon_composite.json
jsonOmon = 'obs4MIPs_Omon_composite.json'
jsonAmon = 'obs4MIPs_Amon.json'
buildList = [
 ['coordinate','https://raw.githubusercontent.com/PCMDI/obs4MIPs-cmor-tables/master/obs4MIPs_coordinate.json'],
#['Omon','https://raw.githubusercontent.com/PCMDI/obs4MIPs-cmor-tables/master/Tables/obs4MIPs_Omon.json'],
 ['Amon','https://raw.githubusercontent.com/PCMDI/obs4MIPs-cmor-tables/master/Tables/obs4MIPs_Amon.json']
 ] ;

# Loop through buildList and create output tables
tmp = readJsonCreateDict(buildList)
for count,table in enumerate(tmp.keys()):
	if table == 'coordinate':
		vars()[table] = tmp[table].get(table)
	else:
		vars()[table] = tmp[table]
del(tmp,count,table) ; gc.collect()

# Now cleanup Amon - These problems require fixes upstream
#Amon['variable_entry'].pop('mc')
#Amon['variable_entry'].pop('phalf')

# Rebuild
table = {}
for count,CV in enumerate(buildList):
    CVName1 = CV[0]
    if CVName1 == 'coordinate':
        table['axis_entry'] = eval(CVName1)
    else:
        keys = eval(CVName1).keys()
        for count in range(len(keys)):
            table[keys[count]] = eval(CVName1).get(keys[count])



outtable = jsonAmon

try:
  os.mkdir('obs4MIPs_CMOR_tables')
except:
  pass

outFile = 'obs4MIPs_CMOR_tables/' + outtable

# Check file exists
if os.path.exists(outFile):
    print 'File existing, purging:',outFile
    os.remove(outFile)
fH = open(outFile,'w')
json.dump(table,fH,ensure_ascii=True,sort_keys=True,indent=4,separators=(',',':'),encoding="utf-8")
fH.close()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Created on Tue Jul 12 16:11:44 2016

Paul J. Durack 12th July 2016

This script generates all json files residing this this subdirectory

PJD 12 Jul 2016     - Started
PJD 13 Jul 2016     - Updated to download existing tables
PJD 14 Jul 2016     - Successfully loaded dictionaries
PJD 15 Jul 2016     - Tables successfully created, coordinates from CMIP6_CVs
PJD 18 Jul 2016     - Generate CVs and tables from CMIP6_CVs and CMIP6-cmor-tables
PJD 19 Jul 2016     - Remove activity_id - no longer in A/O/etc tables
PJD 20 Jul 2016     - Removed target_mip from required_global_attributes
PJD 20 Jul 2016     - Removed source_id
PJD 20 Jul 2016     - Added fx table_id
PJD 20 Jul 2016     - Added readJsonCreateDict function
PJD 20 Jul 2016     - Removed modeling_realm from all variable_entry entries
PJD 27 Sep 2016     - Updated to deal with new upstream data formats
PJD 27 Sep 2016     - Updated tables to "01.beta.30" -> "01.beta.32"
PJD 27 Sep 2016     - Update jsons to include 'identifier' dictionary name (following CMIP6_CVs)
PJD 27 Sep 2016     - Add NOAA-NCEI to institution_id https://github.com/PCMDI/obs4MIPs-cmor-tables/issues/8
PJD 27 Sep 2016     - Correct RSS zip
PJD 28 Sep 2016     - Correct missing 'generic_levels' in Amon table
PJD 29 Sep 2016     - Added ttbr (NOAA-NCEI; Jim Baird [JimBiardCics]) https://github.com/PCMDI/obs4MIPs-cmor-tables/issues/14
                    - TODO:

@author: durack1
"""

#%% Import statements
import gc,json,os,ssl,time
from durolib import readJsonCreateDict

#%% Determine path
homePath = os.path.join('/','/'.join(os.path.realpath(__file__).split('/')[0:-1]))
#homePath = '/export/durack1/git/obs4MIPs-cmor-tables/'
#homePath = '/sync/git/obs4MIPs-cmor-tables/src'
os.chdir(homePath)

#%% Create urllib2 context to deal with lab/LLNL web certificates
ctx                 = ssl.create_default_context()
ctx.check_hostname  = False
ctx.verify_mode     = ssl.CERT_NONE

#%% List target tables
masterTargets = [
 'Amon',
 'Lmon',
 'Omon',
 'SImon',
 'fx',
 'coordinate',
 'frequency',
 'grid',
 'grid_label',
 'grid_resolution',
 'institution_id',
 'mip_era',
 'product',
 'realm',
 'required_global_attributes',
 'table_id'
 ] ;

#%% Tables
tableSource = [
 ['coordinate','https://raw.githubusercontent.com/WCRP-CMIP/CMIP6_CVs/master/CMIP6_coordinate.json'],
 ['fx','https://raw.githubusercontent.com/PCMDI/cmip6-cmor-tables/master/Tables/CMIP6_fx.json'],
 ['grid','https://raw.githubusercontent.com/WCRP-CMIP/CMIP6_CVs/master/CMIP6_grid.json'],
 ['Amon','https://raw.githubusercontent.com/PCMDI/cmip6-cmor-tables/master/Tables/CMIP6_Amon.json'],
 ['Lmon','https://raw.githubusercontent.com/PCMDI/cmip6-cmor-tables/master/Tables/CMIP6_Lmon.json'],
 ['Omon','https://raw.githubusercontent.com/PCMDI/cmip6-cmor-tables/master/Tables/CMIP6_Omon.json'],
 ['SImon','https://raw.githubusercontent.com/PCMDI/cmip6-cmor-tables/master/Tables/CMIP6_SImon.json']
 ] ;

#%% Loop through tables and create in-memory objects
# Loop through tableSource and create output tables
tmp = readJsonCreateDict(tableSource)
for count,table in enumerate(tmp.keys()):
    if 'cmip6_CVs' in tableSource[count][1]:
        vars()[table] = tmp[table].get(table)
    else:
        vars()[table] = tmp[table]
del(tmp,count,table) ; gc.collect()

# Cleanup by extracting only variable lists
for count2,table in enumerate(tableSource):
    tableName = table[0]
    if tableName in ['coordinate','grid']:
        eval(tableName).pop('version_metadata')
        continue
    else:
        eval(tableName).pop('axis_entry')
        eval(tableName)['Header']['table_date'] = time.strftime('%d %B %Y')

# Cleanup realms
Amon['Header']['realm']     = 'atmos'
Lmon['Header']['realm']     = 'land'
Omon['Header']['realm']     = 'ocean'
SImon['Header']['realm']    = 'seaIce'
fx['Header']['realm']       = 'fx'

# Clean out modeling_realm
for jsonName in ['Amon','Lmon','Omon','SImon']:
    dictToClean = eval(jsonName)
    for key, value in dictToClean.iteritems():
        if key == 'Header':
            continue
        for key1,value1 in value.iteritems():
            if 'modeling_realm' in dictToClean[key][key1].keys():
                dictToClean[key][key1].pop('modeling_realm')

# Add new variables
# Variable sponsor - NOAA-NCEI; Jim Baird (JimBiardCics)
Amon['variable_entry'][u'ttbr'] = {}
Amon['variable_entry']['ttbr']['cell_measures'] = 'time: mean'
Amon['variable_entry']['ttbr']['cell_methods'] = 'area: areacella'
Amon['variable_entry']['ttbr']['comment'] = ''
Amon['variable_entry']['ttbr']['dimensions'] = 'longitude latitude time'
Amon['variable_entry']['ttbr']['long_name'] = 'Top of Atmosphere Brightness Temperature'
Amon['variable_entry']['ttbr']['ok_max_mean_abs'] = ''
Amon['variable_entry']['ttbr']['ok_min_mean_abs'] = ''
Amon['variable_entry']['ttbr']['out_name'] = 'ttbr'
Amon['variable_entry']['ttbr']['positive'] = 'time: mean'
Amon['variable_entry']['ttbr']['standard_name'] = 'toa_brightness_temperature'
Amon['variable_entry']['ttbr']['type'] = 'real'
Amon['variable_entry']['ttbr']['units'] = 'K'
Amon['variable_entry']['ttbr']['valid_max'] = '375.0'
Amon['variable_entry']['ttbr']['valid_min'] = '140.0'
                
#%% Coordinate

#%% Frequencies
frequency = ['3hr', '3hrClim', '6hr', 'day', 'decadal', 'fx', 'mon', 'monClim', 'subhr', 'yr'] ;

#%% Grid

#%% Grid labels
grid_label = ['gn', 'gr', 'gr1', 'gr2', 'gr3', 'gr4', 'gr5', 'gr6', 'gr7', 'gr8', 'gr9'] ;

#%% Grid resolutions
grid_resolution = [
 '10 km',
 '100 km',
 '1000 km',
 '10000 km',
 '1x1 degree',
 '25 km',
 '250 km',
 '2500 km',
 '5 km',
 '50 km',
 '500 km',
 '5000 km'
 ] ;

#%% Institutions
tmp = [['institution_id','https://raw.githubusercontent.com/PCMDI/obs4mips-cmor-tables/master/obs4MIPs_institution_id.json']
      ] ;
institution_id = readJsonCreateDict(tmp)
institution_id = institution_id.get('institution_id')
institution_id['institution_id']['NOAA-NCEI'] = 'NOAA\'s National Centers for Environmental Information, Asheville, NC 28801, USA'
institution_id['institution_id']['RSS'] = 'Remote Sensing Systems, Santa Rosa, CA 95401, USA'

 #%% Product
mip_era = ['CMIP6'] ;

#%% Product
product = [
 'composite',
 'remote-sensed',
 'satellite',
 'surface-gridded-insitu',
 'surface-radar'
 ] ;

#%% Realms
realm = [
 'aerosol',
 'atmos',
 'atmosChem',
 'land',
 'landIce',
 'ocean',
 'ocnBgchem',
 'seaIce'
 ] ;

#%% Required global attributes
required_global_attributes = [
 'activity_id',
 'Conventions',
 'creation_date',
 'dataset_version_number',
 'further_info_url',
 'frequency',
 'grid',
 'grid_label',
 'grid_resolution',
 'institution_id',
 'license',
 'mip_era',
 'product',
 'realm',
 'source_id',
 'table_id',
 'tracking_id',
 'variable_id'
 ];

#%% Table IDs
table_id = ['Amon', 'Lmon', 'Omon', 'SImon', 'fx'] ;

#%% Write variables to files
for jsonName in masterTargets:
    # Clean experiment formats
    if jsonName in ['coordinate','grid']: #,'Amon','Lmon','Omon','SImon']:
        dictToClean = eval(jsonName)
        for key, value1 in dictToClean.iteritems():
            for value2 in value1.iteritems():
                string = dictToClean[key][value2[0]]
                if not isinstance(string, list) and not isinstance(string, dict):
                    string = string.strip() ; # Remove trailing whitespace
                    string = string.strip(',.') ; # Remove trailing characters
                    string = string.replace(' + ',' and ')  ; # Replace +
                    string = string.replace(' & ',' and ')  ; # Replace +
                    string = string.replace('   ',' ') ; # Replace '  ', '   '
                    string = string.replace('anthro ','anthropogenic ') ; # Replace anthro
                    string = string.replace('decidous','deciduous') ; # Replace decidous
                    string = string.replace('  ',' ') ; # Replace '  ', '   '
                dictToClean[key][value2[0]] = string
        vars()[jsonName] = dictToClean
    # Write file
    if jsonName in ['Amon','Lmon','Omon','SImon','fx']:
        outFile = ''.join(['../Tables/obs4MIPs_',jsonName,'.json'])
    else:
        outFile = ''.join(['../obs4MIPs_',jsonName,'.json'])
    # Check file exists
    if os.path.exists(outFile):
        print 'File existing, purging:',outFile
        os.remove(outFile)
    if not os.path.exists('../Tables'):
        os.mkdir('../Tables')
    # Create host dictionary
    if jsonName not in ['coordinate','fx','grid','institution_id','Amon','Lmon','Omon','SImon']:
        jsonDict = {}
        jsonDict[jsonName] = eval(jsonName)
    else:
        jsonDict = eval(jsonName)
    fH = open(outFile,'w')
    json.dump(jsonDict,fH,ensure_ascii=True,sort_keys=True,indent=4,separators=(',',':'),encoding="utf-8")
    fH.close()

del(jsonName,outFile) ; gc.collect()

# Validate - only necessary if files are not written by json module



import xlrd

bk=xlrd.open_workbook("Tables_csv/standard_output.xls")#,encoding_override="utf32")

for i in range(bk.nsheets):
    sh = bk._sheet_list[i]
    print 'Treating sheet:',sh.name
    f=open("Tables_csv/"+str(sh.name).split(" ")[0]+'.csv','w')
    for r in range(sh.nrows):
        line=''
        try:
            for c in range(sh.ncols):
                cell=sh.cell(r,c).value
                try:
                    cell=str(cell)
                except:
                    #ok this bit is for Karl showing the problem
                    for (bad,rpl) in  [ (u"\ufb01","fi"),
                                        (u"\u2013","-"),
                                        (u"\u2026","..."),
                                        (u"\xb0",""),
                                        (u"\u201c",'"'),
                                        (u"\u201d",'"'),
                                        ]:
                        while cell.find(bad)>-1:
                            print 'Bad character row %i, col %s, value: %s' % (r+1,chr(c+65),sh.cell(r,c))
                            cell=cell.replace(bad,rpl,1)
                    cell=str(cell)
                cell=cell.replace('"','""')
                if cell.find(",")>-1 or cell.find('"')>-1:
                    line+='"%s",' % cell
                else:
                    line+='%s,' % cell
        except Exception,err:
            print 'Error row: %i, col: %i, value: %s' % (r,c,sh.cell(r,c))
            print err
            line+=','
        line=line[:-1]
        print >> f,line
    print >> f,"\n"
    f.close()
            



import cmor
import cdms2 as cdm
import numpy as np
import pdb
import sys
import argparse
import MV2

inp = '../Tables'
#inp = '.'
input_json = './prw_rss_mo_input.json' 
input_data_path = '/clim_obs/orig/data/RSS/rss_ssmi_pw_v06.6-demo.nc' 
input_var_name = 'prw'
cmor_table = './obs4MIPs_CMOR_tables/obs4MIPs_Amon.json' 
#cmor_table = 'obs4MIPs_Amon.json'
output_units = 'kg m-2'
output_var_name = 'prw'

### BETTER IF THE USER DOES NOT HAVE TO CHANGE ANYTHING BELOW THIS LINE... 
 
#%% Process variable (with time axis)
cmor.setup(inpath=inp,netcdf_file_action=cmor.CMOR_REPLACE_4)
#cmor.dataset_json('./drive_obs4MIPs.json') ; # Update contents of this file to set your global_attributes
cmor.dataset_json(input_json) ; # Update contents of this file to set your global_attributes

f       = cdm.open(input_data_path)
d       = f(input_var_name)
lat     = d.getLatitude()
lon     = d.getLongitude()
time    = d.getTime()
time   = d.getAxis(0)
time.axis = 'T'

#cmor.set_cur_dataset_attribute('history',f.history) ; # Force local file attribute as history
#table   = '../obs4MIPs_Omon_composite.json' ; # Amon,Lmon,Omon,SImon
table   = cmor_table ; # Amon,Lmon,Omon,SImon

obs4MIPsAmonID = cmor.load_table(table) ; # Load target table (above), axis info (coordinates, grid*) and CVs
axes    = [ {'table_entry': 'time',
             'units': time.units,   #'days since 1870-01-01',
             },
             {'table_entry': 'latitude',
              'units': 'degrees_north',
              'coord_vals': lat[:],
              'cell_bounds': lat.getBounds()},
             {'table_entry': 'longitude',
              'units': 'degrees_east',
              'coord_vals': lon[:],
              'cell_bounds': lon.getBounds()},
          ]
axis_ids = list()
for axis in axes:
    axis_id = cmor.axis(**axis)
    axis_ids.append(axis_id)

#pdb.set_trace()

d.units = output_units 
varid   = cmor.variable(output_var_name,d.units,axis_ids,missing_value=d.missing)
values  = np.array(d[:],np.float32)
cdm.setAutoBounds('on')   # PJG  CAUTION WITH THIS ONE! 

cmor.set_deflate(varid,1,1,1) ; # shuffle=1,deflate=1,deflate_level=1 ; CMOR 3.0.6+
cmor.write(varid,values,time_vals=time[:],time_bnds=time.getBounds()) ; # Write variable with time axis
f.close()
cmor.close()




## Function to check validity of CMOR CMIP5 output

import httplib

class TableBadName(Exception):
    pass
class TableBadDate(Exception):
    pass
class TableBadMD5(Exception):
    pass

class CMORTables:
    def __init__(self,name,prefix,url="uv-cdat.llnl.gov"):
        self.repo_url=url
        self.repo_name=name
        self.repo_prefix=prefix
        self.H=httplib.HTTPConnection(self.repo_url)

    def splitTableString(self,str):
        sp=str.split()
        table = sp[1]
        date=" ".join(sp[2:5])[1:-1].strip()
        md5 = sp[-1]
        if len(md5)!=32:
            md5=None
        return table,date,md5

    def preprocess(self,table,date=None,md5=None):
        if date is None and md5 is None:
            table,date,md5 = self.splitTableString(table)
        return table,date,md5

    def fetchLatestTable(self,table):
        self.H.request("GET","/gitweb/?p=%s.git;a=blob_plain;f=Tables/%s_%s;hb=HEAD" % (self.repo_name,self.repo_prefix,table))
        r = self.H.getresponse()
        return r.read()


    def fetchATable(self,table,commit):
        self.H.request("GET","/gitweb/?p=%s.git;a=blob_plain;f=Tables/%s_%s;h=%s" % (self.repo_name,self.repo_prefix,table,commit))
        r=self.H.getresponse()
        return r.read()

    def fetchTable(self,table,date=None):
        table,date,md5 = self.preprocess(table,date)
        self.checkTable(table,date)
        #First check if it is the latest table
        t=self.fetchLatestTable(table)
        j=t.find("\ntable_date:")
        tdate = t[j+12:j+100]
        tdate = tdate.split("\n")[0].split("!")[0].strip()
        if tdate == date:
            return t
        # Ok now fetch the history
        self.H.request("GET","/gitweb/?p=%s.git;a=history;f=Tables/%s_%s;hb=HEAD" % (self.repo_name,self.repo_prefix,table))
        r = self.H.getresponse().read()
        for l in r.split("\n"):
            i= l.find(";hp=")
            if i>-1:
                commit=l[i+4:i+44]
                t = self.fetchATable(table,commit)
                j=t.find("\ntable_date:")
                tdate = t[j+12:j+100]
                tdate = tdate.split("\n")[0].split("!")[0].strip()
                if tdate == date:
                    break
        return t

    def checkTable(self,table,date=None,md5=None):
        table,date,md5 = self.preprocess(table,date,md5)
        self.H.request("GET","/gitweb/?p=%s.git;a=blob_plain;f=Tables/md5s;hb=HEAD" % self.repo_name)
        r = self.H.getresponse()
        md5Table = eval( r.read())[self.repo_prefix]
        table = md5Table.get(table,None)
        if table is None:
            raise TableBadName("Invalid Table name: %s" % table)
        dateMd5 = table.get(date,None)
        if dateMd5 is None:
            raise TableBadDate("Invalid Table date: %s" % date)
        if md5 is not None and md5!=dateMd5:
            raise TableBadMD5("Invalid Table md5: %s" % md5)
        return

if __name__=="__main__":
    repo_name = "cmip5-cmor-tables"
    repo_url = "uv-cdat.llnl.gov"
    repo_prefix="CMIP5"
    Tables = CMORTables(repo_name,repo_prefix,repo_url)
    t = Tables.fetchTable("cfSites","27 April 2011")
    #t = Tables.fetchATable("cfSites","6dc5daf0e185a4523c9eb297014fcd5b696173b9")
    t=Tables.fetchLatestTable("cfSites")
    print t


## import cdms2
## f=cdms2.open("/git/cmor/CMIP5/output/ukmo/HadCM3/piControl/monClim/ocean/difvso/r1i1p1/difvso_Oclim_HadCM3_piControl_r1i1p1_185001-184912_clim.nc")
## tid = f.table_id

## checkTable(tid)
## try:
##     checkTable("Oclim","11 April 2011")
## except:
##     print "darn it should have worked!"

## checkTable("Table Oclim (11 April 2011) 02c858e13f41cc2d92dde421ff54f504")
## try:
##     checkTable("Table Oclim (11 April 2011) 02c858e13f41cc2d92dde421ff54f504")
## except:
##     print "oh it should have worked"
    
## try:
##     checkTable("Table Oclim (11 April 2011)")
## except:
##     print "oh it should have worked"
    
## try:
##     checkTable("Oclim","11 April 2011","5b69b1f13c586a193e3e7da9207d9474")
## except TableBadMD5:
##     print "OK it did failed with bad md5 as expected"
## except:
##     print "Bad exception raised"
## try:
##     checkTable("Oclim","12 April 2001")
## except TableBadDate:
##     print "OK it did failed with bad date as expected"
## except:
##     print "darn worng exception raised for bad date"

## try:
##     checkTable("Oclimy","11 April 2011")
## except TableBadName:
##     print "OK it did failed with bad name as expected"
## except:
##     print "darn worng exception raised for bad name"




import hashlib

import os

ls=os.popen("ls Tables/CMIP5*")

if os.path.exists("Tables/md5s"):
    f=open("Tables/md5s")
    tbls=eval(f.read())
    f.close()
else:
    tbls={}

for l in ls:
    fnm = l.strip()
    f=open(fnm)
    t=f.read()
    i=t.find("table_id:")
    id=t[i+9:].split("\n")[0].split()[-1]
    i=t.find("table_date:")
    date = t[i+11:].split("\n")[0].split("!")[0].strip()
    i=t.find("project_id:")
    pid = t[i+11:].split("\n")[0].split("!")[0].strip()
    md5=hashlib.md5(t)
    md5=md5.hexdigest()
    print fnm,pid,id,date,md5
    pdic = tbls.get(pid,{})
    tdic = pdic.get(id,{})
    if tdic.has_key(date):
        print "WARNING: Replacing exisiting date for date %s in table %s of project %s" % (date,id,pid)
    tdic[date]=md5
    pdic[id]=tdic
    tbls[pid]=pdic

f=open("Tables/md5s","w")
f.write(repr(tbls))
f.close()



import sys,time,os,genutil,numpy


prefix = "CMIP5"

general = """cmor_version: 2.6 ! minimum version of CMOR that can read this table
cf_version:   1.4         ! version of CF that output conforms to
project_id:   %s  ! project id
table_date:   %s ! date this table was constructed

missing_value: 1.e20      ! value used to indicate a missing value
                          !   in arrays output by netCDF as 32-bit IEEE 
                          !   floating-point numbers (float or real)

baseURL: http://cmip-pcmdi.llnl.gov/CMIP5/dataLocation 
product: output

required_global_attributes: creation_date tracking_id forcing model_id parent_experiment_id parent_experiment_rip branch_time contact institute_id ! space separated required global attribute 

forcings:   N/A Nat Ant GHG SD SI SA TO SO Oz LU Sl Vl SS Ds BC MD OC AA

expt_id_ok: '10- or 30-year run initialized in year XXXX' 'decadalXXXX'
expt_id_ok: 'volcano-free hindcast initialized in year XXXX' 'noVolcXXXX'
expt_id_ok: 'prediction with 2010 volcano' 'volcIn2010'
expt_id_ok: 'pre-industrial control' 'piControl'
expt_id_ok: 'historical' 'historical'
expt_id_ok: 'historical extension' 'historicalExt'
expt_id_ok: 'other historical forcing' 'historicalMisc'
expt_id_ok: 'mid-Holocene' 'midHolocene'
expt_id_ok: 'last glacial maximum' 'lgm'
expt_id_ok: 'last millennium' 'past1000'
expt_id_ok: 'RCP4.5' 'rcp45'
expt_id_ok: 'RCP8.5' 'rcp85'
expt_id_ok: 'RCP2.6' 'rcp26'
expt_id_ok: 'RCP6' 'rcp60'
expt_id_ok: 'ESM pre-industrial control' 'esmControl'
expt_id_ok: 'ESM historical' 'esmHistorical'
expt_id_ok: 'ESM RCP8.5' 'esmrcp85'
expt_id_ok: 'ESM fixed climate 1' 'esmFixClim1'
expt_id_ok: 'ESM fixed climate 2' 'esmFixClim2'
expt_id_ok: 'ESM feedback 1' 'esmFdbk1'
expt_id_ok: 'ESM feedback 2' 'esmFdbk2'
expt_id_ok: '1 percent per year CO2' '1pctCO2'
expt_id_ok: 'abrupt 4XCO2' 'abrupt4xCO2'
expt_id_ok: 'natural-only' 'historicalNat'
expt_id_ok: 'GHG-only' 'historicalGHG'
expt_id_ok: 'AMIP' 'amip'
expt_id_ok: '2030 time-slice' 'sst2030'
expt_id_ok: 'control SST climatology' 'sstClim'
expt_id_ok: 'CO2 forcing' 'sstClim4xCO2'
expt_id_ok: 'all aerosol forcing' 'sstClimAerosol'
expt_id_ok: 'sulfate aerosol forcing' 'sstClimSulfate'
expt_id_ok: '4xCO2 AMIP' 'amip4xCO2'
expt_id_ok: 'AMIP plus patterned anomaly' 'amipFuture'
expt_id_ok: 'aqua planet control' 'aquaControl'
expt_id_ok: '4xCO2 aqua planet' 'aqua4xCO2'
expt_id_ok: 'aqua planet plus 4K anomaly' 'aqua4K'
expt_id_ok: 'AMIP plus 4K anomaly' 'amip4K'

""" % (prefix,time.strftime("%d %B %Y"))


olexps="""
expt_id_ok: 'anthropogenic-only' 'historicalAnt'
expt_id_ok: 'anthropogenic sulfate aerosol direct effect only' 'historicalSD'
expt_id_ok: 'anthropogenic sulfate aerosol indirect effect only' 'historicalSI'
expt_id_ok: 'anthropogenic sulfate aerosol only' 'historicalSA'
expt_id_ok: 'tropospheric ozone only' 'historicalTO'
expt_id_ok: 'stratospheric ozone' 'historicalSO'
expt_id_ok: 'ozone only' 'historicalOz'
expt_id_ok: 'land-use change only' 'historicalLU'
expt_id_ok: 'solar irradiance only' 'historicalSl'
expt_id_ok: 'volcanic aerosol only' 'historicalVl'
expt_id_ok: 'sea salt only' 'historicalSS'
expt_id_ok: 'dust' 'historicalDs'
expt_id_ok: 'black carbon only' 'historicalBC'
expt_id_ok: 'mineral dust only' 'historicalMD'
expt_id_ok: 'organic carbon only' 'historicalOC'
expt_id_ok: 'anthropogenic aerosols only' 'historicalAA'
"""

#realm:	      %s
#table_id:     Table %s    ! table id


axis_tmpl  = """
!============
axis_entry: %(CMOR dimension)
!============
!----------------------------------
! Axis attributes:
!----------------------------------
standard_name:    %(standard name)
units:            %(units)
axis:             %(axis)             ! X, Y, Z, T (default: undeclared)
positive:         %(positive)         ! up or down (default: undeclared)
long_name:        %(long name)
!----------------------------------
! Additional axis information:
!----------------------------------
out_name:         %(output dimension name)
valid_min:        %(valid_min)         
valid_max:        %(valid_max) 
stored_direction: %(stored direction)
formula:          %(formula)
z_factors:        %(z_factors)
z_bounds_factors: %(z_factor_bounds)
tolerance:        %(tol_on_requests: variance from requested values that is tolerated)
type:             %(type)
requested:        %(requested)        ! space-separated list of requested coordinates 
requested_bounds: %(bounds_ requested) ! space-separated list of requested coordinate bounds
value:            %(value)            ! of scalar (singleton) dimension 
bounds_values:    %(bounds _values)    ! of scalar (singleton) dimension bounds
must_have_bounds: %(bounds?)
index_only:       %(index axis?)
climatology:      %(climatology)
coords_attrib:    %(coords_attrib)
must_call_cmor_grid: %(grid?)
!----------------------------------
!
"""


var_tmpl = """!============
variable_entry:    %(CMOR variable name)
!============
modeling_realm:    %(realm)
!----------------------------------
! Variable attributes:
!----------------------------------
standard_name:     %(standard name)
units:             %(unformatted units)
cell_methods:      %(cell_methods)
cell_measures:     %(cell_measures)
long_name:         %(long name)
comment:           %(comment)
!----------------------------------
! Additional variable information:
!----------------------------------
dimensions:        %(CMOR dimensions)
out_name:          %(output variable name)
type:              %(type)
positive:          %(positive)
valid_min:         %(valid min)
valid_max:         %(valid max)
ok_min_mean_abs:   %(mean absolute min)
ok_max_mean_abs:   %(mean absolute max)
flag_values:       %(flag_values)
flag_meanings:     %(flag_meanings)
!----------------------------------
!
"""

def process_a_line(line):
    #print line
    if line.find("usiiiiually,")>-1:
        debug = True
        print
        print 'NEW STUFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF'
        print
    else:
        debug=False
    iadd = line.find("$$$$add at surface$$$")
    if iadd>-1:
        line=line.replace("$$$$add at surface$$$","")
    line=line.replace("\r\n","\n") # dos character 
    line = line.replace('""','"$$$')
    sps = line .split(",")
    sp=[]
    st=""
    while (len(sps)>0):
        s = sps.pop(0)
        if debug:
            print 's:',s
        if len(s)>0 and s[0]=='"' and s[-1]!='"':
            if debug: print 'inthere'
            s=s[1:]
            s2=sps.pop(0)
            if debug: print 's2:',s2
            while len(s2)==0 or s2[-1]!='"':
                s = ','.join([s,s2])
                #print sps
                s2=sps.pop(0)
            else:
                s = ','.join([s,s2[:-1]])                
        sp.append(s.replace("$$$",'"'))
    return sp,iadd


def process_template(tmpl,cnames,cols,voids={},minmax={},iadd=-1):
    F = genutil.StringConstructor(tmpl)

    keys = F.keys()
    match = 0
    for c in cnames:
        if c in keys:
            match+=1
            indx = cnames.index(c)
##             print ' matched at %i' % indx ,
            if indx<len(cols):
                val = cols[indx]
            else:
                val = ""
            if  val.strip()=='time2':
                setattr(F,"climatology","yes")
                if "climatology" in keys: keys.remove("climatology")
            if c=="long name" and iadd!=-1:
                val+=" at Surface"
            if val.strip()!="":
                ## if c=="valid max":
                ##     print 'ok we havew a val max predefined at:',cols
                if c in ['units','unformatted units'] and val=='1.0':
                    val='1'
                setattr(F,c,val)
                keys.remove(c)
##                 print
##             else:
##                 print ' but empty'
##         else:
##             print
    #print 'Keys:',keys
    if "CMOR dimension" in keys:
        print 'Keys:',keys
        print 'cnames:',cnames
        raise "crap"
    nstd  = 3.0
    pmean = 0.2
    ve = getattr(F,"CMOR variable name","yep not that guy") 
    if ve in minmax.keys():
        if 'valid min' in keys:
        #ok let's see if we can figure this one out
            mnmx = minmax[ve]
            val=1.e20
            std=0.
            for mlev in mnmx.keys():
                mn=mnmx[mlev]['Min']
                val = min(mn['min'],val)
                std +=mn['std']
            std/=len(mnmx.keys())
            if numpy.allclose(std,0.):
                std=val*pmean
            delta=max(nstd*std,abs(val*.05))
            setattr(F,"valid min","%.4g" % (val-delta))
            keys.remove("valid min")
        if 'valid max' in keys:
        #ok let's see if we can figure this one out
            mnmx = minmax[ve]
            val=-1.e20
            std=0.
            for mlev in mnmx.keys():
                mn=mnmx[mlev]['Max']
                val = max(mn['max'],val)
                std +=mn['std']
            std/=len(mnmx.keys())
            if numpy.allclose(std,0.):
                std=val*pmean
            delta=max(nstd*std,abs(val*.05))
            setattr(F,"valid max","%.4g" % (val+delta))
            keys.remove("valid max")
        if "mean absolute min" in keys:
            mnmx = minmax[ve]
            val=1.e20
            std=0.
            for mlev in mnmx.keys():
                aavg = mnmx[mlev]['AAvg']
                val = min(aavg['min'],val)
                std+=aavg['std']
            std/=len(mnmx.keys())
            if numpy.allclose(std,0.):
                std=val*pmean
            delta=max(nstd*std,abs(val*.05))
            setattr(F,"mean absolute min","%.4g" % (val-delta))
            keys.remove("mean absolute min")
        if "mean absolute max" in keys:
            mnmx = minmax[ve]
            val=-1.e20
            std=0.
            for mlev in mnmx.keys():
                aavg = mnmx[mlev]['AAvg']
                val = max(aavg['max'],val)
                std+=aavg['std']
            std/=len(mnmx.keys())
            if numpy.allclose(std,0.):
                std=val*pmean
            delta=max(nstd*std,abs(val*.05))
            setattr(F,"mean absolute max","%.4g" % (val+delta))
            keys.remove("mean absolute max")

        ### Need to add lines for absolute mean min/max
                
    for k in keys:
        setattr(F,k,"!CRAP WE NEED TO REMOVE THAT LINE")

    ## Now generates
    out = F()
    sp = out.split("\n")
    lines = []
    for l in sp:
        if l.find("!CRAP WE NEED TO REMOVE THAT LINE")>-1:
            continue
        lines.append(l)
    out = "\n".join(lines)
            
##     print 'We got: %i matches' % match
    # fixes Karl input bug
    out = out.replace("..",".")

    #Ok now check the void thing
    for kw in voids.keys():
        v = getattr(F,kw,"we keep").strip()
        vals = voids[kw]
        if not isinstance(vals,(list,tuple)):
            vals = [vals,]
        for V in vals:
            if V == v:
                print 'Skipping:',sp
                out = ""
    return out

def get_interval(tbnm):
    if tbnm.lower().find("mon")>-1:
        interval = 30.
    elif tbnm.lower().find('clim')>-1:
        interval = 30.
    elif tbnm.lower().find('aero')>-1:
        interval = 30.
    elif tbnm.lower().find('yr')>-1:
        interval = 365.
    elif tbnm.lower().find('da')>-1:
        interval = 1.
    elif tbnm.lower().find("hr")>-1:
        indx = tbnm.lower().find("hr")
        interval = float(tbnm[indx-1])/24.
    elif tbnm.lower().find("cfoff")>-1:
        interval = 30.
    elif tbnm.lower().find("cfsites")>-1:
        interval = 25./(24.*60.)
    elif tbnm.lower().find("min")>-1:
        interval = float(tbnm[2:tbnm.find("min")])/1440.
    else:
        interval = 0.
    return interval

def create_table_header(tbnm, table_file, dims_file, fqcy):
    #First of All create the header
    fnm = "Tables/" + prefix + '_'+tbnm
    fo = open(fnm,'w')
    print >> fo, "table_id: Table %s" % tbnm
    if tbnm.lower().find("cfsites")>-1:
        print >> fo, "!\n! WARNING: Although this table define variables as function of longitude and latitude\n!          the user MUST use a 'grid' (using the 'site' index in CMIP5_grids)\n!          in order to define correctly the location of each site.\n!"
    realm = None
    if tbnm[0]=='O':
        realm = "ocean"
    elif tbnm[0]=='A':
        realm = 'atmos'
    elif tbnm[0]=='L':
        realm = 'land'
    else:
        realm = "atmos"


    print >> fo, "modeling_realm: %s\n" % realm
    print >> fo, "frequency: %s\n" % fqcy
    print >> fo, general

    # looking for approx interval, ASSUMING UNITS ARE IN DAYS SINCE
    print 'Table nm:',tbnm,'__________________________________________'
    interval = get_interval(tbnm)
    #print 'Interval:',interval,'________________________'
    print >> fo, """approx_interval:  %f     ! approximate spacing between successive time
                          !   samples (in units of the output time 
                          !   coordinate.""" % interval
    if tbnm == "cfSites":
        print >> fo,"""approx_interval_warning:  0.25 ! Level at which a warning is issued because the time axis values are too far appart from the official interval
approx_interval_error:  0.75 ! Level at which an error is issued because the time axis values are too far appart from the official interval"""


    D = open(dims_file)
    dlines = D.readlines()[2:]
    i=0
    while dlines[i].strip()=="":
        i+=1
    dlines=dlines[i:]
    cnms = dlines[0].split(',')
    for i in  range(len(cnms)):
        cnms[i]=cnms[i].strip()


    addLines = False
    generic_levels = []
    for l in dlines[1:]:
        sp, iadd = process_a_line(l)
        foundnm = False
        for snm in sp[0].split(","):
            if tbnm == snm.strip():
                foundnm  = True
        if foundnm:
            if l.find("alevel")>-1:
                addLines = True
                zlevel_name = 'alevel'
                file_add = 'Tables_csv/add_dims.txt'
                if not "alevel" in generic_levels:
                    generic_levels.append("alevel")
            elif l.find("alev1")>-1:
                addLines = True
                zlevel_name = 'alevel'
                file_add = 'Tables_csv/add_dims.txt'
                if not "alev1" in generic_levels:
                    generic_levels.append("alev1")
            elif l.find("alevhalf")>-1:
                addLines = True
                zlevel_name = 'alevel'
                file_add = 'Tables_csv/add_dims.txt'
                if not "alevhalf" in generic_levels:
                    generic_levels.append("alevhalf")
            elif l.find("olevel")>-1:
                addLines = True
                zlevel_name = 'olevel'
                file_add = 'Tables_csv/add_dims2.txt'
                if tbnm=='fx':
                    file_add= "Tables_csv/add_dims2_notime.txt"
                if not "olevel" in generic_levels:
                    generic_levels.append("olevel")
    if generic_levels!=[]:
        print >> fo, "\ngeneric_levels:   %s" % " ".join(generic_levels)
    for l in dlines[1:]:
        sp, iadd = process_a_line(l)
        foundnm = False
        for snm in sp[0].split(","):
            if tbnm == snm.strip():
                foundnm  = True
        if iadd!=-1: print 'iadd is:',iadd
        if foundnm:
            if l.find("alevel")>-1 or l.find("alev1")>-1 or l.find("alevhalf")>-1 or l.find("olevel")>-1:
                pass
            else:
                print >> fo, process_template(axis_tmpl,cnms,sp,iadd=iadd)
    if addLines is True:
        print 'adding:',file_add,'to',tbnm
        tmpf=open(file_add)
        lns=tmpf.readlines()
        tmpf.close()
        if file_add == 'Tables_csv/add_dims.txt':
            if not tbnm in ['aero','cfMon']:
                lns=lns[:-20]
        lns=''.join(lns)
        lns=lns.replace("zlevel",zlevel_name)
        if tbnm.find("Oclim")>-1:
            lns=lns.replace("dimensions:      longitude latitude time","dimensions:      longitude latitude time2")
        if tbnm.find("cfSite")>-1:
            lns=lns.replace("longitude latitude","site")
        print >> fo, lns

    return fo


def create_table(table_file, dims_file,minmax={}):
    tables = {}
    foundnm= False
    D = open(table_file)
    dlines = D.readlines()
    ivar = -2
    if len(dlines)==1:
        dlines = dlines[0].split('\r')
    #This bit insert a table
    dlines2=[]
    for i in range(len(dlines)):
        if dlines[i].find("include Amon 2D")>-1:
            #print 'Ok adding amon_2D',table_file,table_file[-11:-4]
            f=open("Tables_csv/Amon_2D.csv")
            add_lines = f.readlines()
            if table_file[-11:-4] in ['cfSites','v/cf3hr']:
                tmplines=[]
                for aline in add_lines:
                    a_line =aline.strip()
                    if len(a_line)==0:
                        continue
                    if a_line[-1]!=',' : a_line=a_line+','
                    sp, iadd = process_a_line(a_line)
                    ## for i2 in range(len(sp)):
                    ##     if sp[i2]=='tasmax' : print i2,sp[i2]
                    if len(sp)>15 and 'time' in sp[16]:
                        sp[16]=sp[16].replace('time','time1')
                    if table_file[-11:-4]=='cfSites':
                        sp[16]=sp[16].replace("longitude latitude","site")
                    for i in range(len(sp)):
                        if sp[i].find(",")>-1:
                            sp[i]='"%s"' % sp[i]
                    myline = ','.join(sp)
                    myline = myline.replace("time: mean","time: point")
                    if len(sp)>4 and sp[5] in ['tasmin','tasmax']:
                        print 'Skipping tasmin/max'
                        pass
                    else:
                        tmplines.append(myline)
                add_lines = tmplines
            dlines2=dlines2+add_lines
        elif dlines[i].find("include Oyr")>-1:
            f=open("Tables_csv/Oyr_tracer.csv")
            addedlines=[]
            for l in f.xreadlines():
                l=l.replace("olevel time","time depth0m")
                l=l.replace("olevel","depth0m")
                l+='$$$$add at surface$$$'
                addedlines.append(l)
            dlines2=dlines2+addedlines
        else:
            dlines2.append(dlines[i])
    dlines=dlines2
    for l in dlines:
        sp,iadd = process_a_line(l)
        if len(sp)>15 and table_file[-11:-4]=='cfSites':
            sp[16]=sp[16].replace("longitude latitude","site")
        if 0<=sp[0].find("CMOR Table")<=1 and foundnm == False: # line that will give us the table name
            i=1
            while sp[i].strip()=="":
                i+=1
            tbnm = sp[i].strip()
            fqcy = sp[i+1].strip()
            foundnm = True
        if sp[0] == 'priority':
            cnms = sp
            for i in  range(len(cnms)):
                cnms[i]=cnms[i].strip()
            ivar = cnms.index("CMOR variable name")
            continue
        if ivar!=-2 and len(sp)>ivar and sp[ivar].strip()!="":
            if tbnm in tables.keys():
                fo = tables[tbnm]
            else: # New table
                print 'Creating table:',tbnm
                fo = create_table_header(tbnm,table_file,dims_file,fqcy)
                tables[tbnm]=fo
            if get_interval(tbnm)!=30.:
                print 'Interval is:',get_interval(tbnm)
                minmaxpassed={}
            else:
                minmaxpassed=minmax
                for v in ["sltovovrt","sltovgyre"]:
                    try:
                        del(minmaxpassed[v])
                    except:
                        pass

            print "Sending columns: ",cnms
            print >> fo, process_template(var_tmpl,cnms,sp,{'CMOR variable name':['?','0','0.0']},minmax=minmaxpassed,iadd=iadd)
    print 'Created tables:',tables.keys()
                
        

    

if __name__== "__main__" :
    import extract_min_max
    import pickle
    f=open("Tables_csv/minmax.pickled")
    minmax = pickle.load(f)
    if len(sys.argv)>2:
        dims_table = sys.argv[2]
    else:
        dims_table = 'Tables_csv/dims.csv'

    if len(sys.argv)>1:
        print sys.argv
        create_table(sys.argv[1],dims_table,minmax=minmax)
    else:
        tables_nms = """
Tables_csv/cfOff.csv    Tables_csv/Omon.csv     Tables_csv/Amon.csv
Tables_csv/cfMon.csv    Tables_csv/Oclim.csv    Tables_csv/6hrPlev.csv
Tables_csv/fx.csv       Tables_csv/cfDay.csv    Tables_csv/OImon.csv    
Tables_csv/cf3hr.csv    Tables_csv/Lmon.csv     Tables_csv/3hr.csv
Tables_csv/day.csv      Tables_csv/aero.csv     Tables_csv/LImon.csv
Tables_csv/cfSites.csv  Tables_csv/Oyr.csv      Tables_csv/6hrLev.csv
""".split()

##         tables_nms = """Tables_csv/3hr.csv      Tables_csv/amon.csv     Tables_csv/cfMon.csv    Tables_csv/oclim.csv
## Tables_csv/6hrLev.csv   Tables_csv/cfsites.csv  Tables_csv/cfOff.csv    Tables_csv/fx.csv       Tables_csv/olmon.csv
## Tables_csv/6hrPlev.csv  Tables_csv/cf3hr.csv    Tables_csv/llmon.csv    Tables_csv/omon.csv
## Tables_csv/aero.csv     Tables_csv/cfDa.csv     Tables_csv/da.csv       Tables_csv/lmon.csv     Tables_csv/oyr.csv
## """.split()
        for nm in tables_nms:
            print 'Processing:',nm
            create_table(nm,dims_table,minmax=minmax)



import cmor,numpy
from cmor.check_CMOR_compliant import readTable

tables = """CMIP5_3hr     CMIP5_6hrPlev CMIP5_LImon   CMIP5_OImon   CMIP5_Omon    CMIP5_aero    CMIP5_cfDay   CMIP5_cfOff   CMIP5_day     CMIP5_grids CMIP5_6hrLev  CMIP5_Amon    CMIP5_Lmon    CMIP5_Oclim   CMIP5_Oyr     CMIP5_cf3hr   CMIP5_cfMon   CMIP5_cfSites CMIP5_fx"""
tables = """CMIP5_3hr     CMIP5_6hrPlev CMIP5_LImon   CMIP5_OImon   CMIP5_Omon    CMIP5_aero    CMIP5_cfDay   CMIP5_cfOff   CMIP5_day      CMIP5_6hrLev  CMIP5_Amon    CMIP5_Lmon    CMIP5_Oclim   CMIP5_Oyr    CMIP5_cfMon    CMIP5_fx"""

for t in tables.split():
    print 'Table:',t
    tnm = "Tables/%s" % t
    tbl = readTable(tnm)
    vr = tbl['variable']
    ax = tbl['axis']
    g = tbl['general']
    cmor.setup(inpath='Tables',set_verbosity=cmor.CMOR_QUIET,
               netcdf_file_action = cmor.CMOR_REPLACE,exit_control=cmor.CMOR_EXIT_ON_MAJOR)
    cmor.dataset('pre-industrial control', 'ukmo', 'HadCM3', '360_day',
                 institute_id = 'ukmo',
                 model_id = 'HadCM3',
                 forcing="TO",
                 contact="Derek Jeter",
                 history = 'some global history',
                 parent_experiment_id="lgm",branch_time=0)
    cmor.load_table(tnm)
    dims={}
    print '\tDimensions:'
    for a in ax:
        A=ax[a]
        print '\t\t',a,A.get('type','notype')
        mn = float(A.get("valid_min",-1.e20))
        mx = float(A.get("valid_max",1.e20))
        val = (mn+mx)/2.
        try:
            val = numpy.array([float(A.get("value",val)),])
        except:
            val = [A.get("value"),]
        if A.has_key("requested"):
            val=[]
            isnum=True
            for v in A['requested']:
                try:
                    val.append(float(v))
                except:
                    isnum=False
                    val.append(v)
            if isnum: val=numpy.array(val)
        kargs = {'table_entry':a,'units':A.get('units','1'),'coord_vals':val}
        if A.get("must_have_bounds","no")=="yes":
            if A.get("requested_bounds",None) is not None:
                rb = A['requested_bounds']
                print rb, 'vs', A.get('requested',None)
                bnds=[]
                rsp=rb.split()
                if len(rsp)>2 and rsp[1]!=rsp[2]:
                    for b in rsp:
                        print b
                        bnds.append(float(b))                        
                else:
                    for b in rsp[::2]:
                        print b
                        bnds.append(float(b))
                    bnds.append(float(rsp[-1]))
                kargs['cell_bounds']=numpy.array(bnds)
                if A.get('requested',None) is None:
                    vals=[]
                    for i in range(len(bnds)-1):
                        vals.append((bnds[i]+bnds[i+1])/2.)
                    vals=numpy.array(vals)
                    kargs['coord_vals']=vals
            elif A.get("bounds_values",None) is None:
                if A.get("axis","N")=='T':
                    kargs['cell_bounds']=numpy.array([val[0],val[0]+float(g['approx_interval'])])
                else:
                    if A.get("stored_direction","increasing")=="increasing":
                        kargs['cell_bounds']=numpy.array([val[0]-.1*abs(val[0]),val[0]+.1*abs(val[0])])
                    else:
                        kargs['cell_bounds']=numpy.array([val[0]+.1*abs(val[0]),val[0]-.1*abs(val[0])])
            else:
                b=A["bounds_values"].split()
                kargs['cell_bounds']=numpy.array([float(b[0]),float(b[1])])
        dims[a]=cmor.axis(**kargs)
    print '\tVariables:'
    for v in vr.keys():
        Vr=vr[v]
        print '\t\t',v,':',
        try:
            print Vr['dimensions']
        except:
            print 'No dims, skipped'
            continue
        vdims=[]
        for d in Vr['dimensions']:
            if d=='olevel':
                vdims.append(dims['depth_coord'])
            elif d in g.get('generic_levels',"").split():
                vdims.append(dims['smooth_level'])
            else:
                vdims.append(dims[d])
        kargs={}
        if Vr.has_key("positive"):
            kargs['positive']=Vr['positive']
        V=cmor.variable(table_entry=v,units=Vr.get('units','1'),axis_ids=numpy.array(vdims),**kargs)
    cmor.close()
print 'Done'



# Loop through input tables
def readJsonCreateDict(buildList):
    '''
    Documentation for readJsonCreateDict(buildList):
    -------
    The readJsonCreateDict() function reads web-based json files and writes
    their contents to a dictionary in memory

    Author: Paul J. Durack : pauldurack@llnl.gov

    Usage:
    ------
        >>> from runCMOR3 import readJsonCreateDict
        >>> readJsonCreateDict(['Omon','https://raw.githubusercontent.com/PCMDI/obs4MIPs-cmor-tables/master/Tables/obs4MIPs_Omon.json'])

    Notes:
    -----
        ...
    '''
    # Test for list input of length == 2
    if len(buildList[0]) != 2:
        print 'Invalid inputs, exiting..'
        sys.exit()
    # Create urllib2 context to deal with lab/LLNL web certificates
    ctx                 = ssl.create_default_context()
    ctx.check_hostname  = False
    ctx.verify_mode     = ssl.CERT_NONE
    # Iterate through buildList and write results to jsonDict
    jsonDict = {}       
    for count,table in enumerate(buildList):
        print 'Processing:',table[0]
        # Read web file
        jsonOutput = urllib2.urlopen(table[1], context=ctx)
        tmp = jsonOutput.read()
        vars()[table[0]] = tmp
        jsonOutput.close()
        # Write local json
        tmpFile = open('tmp.json','w')
        tmpFile.write(eval(table[0]))
        tmpFile.close()
        # Read local json
        vars()[table[0]] = json.load(open('tmp.json','r'))
        os.remove('tmp.json')
        jsonDict[table[0]] = eval(table[0]) ; # Write to dictionary
    
    return jsonDict






#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Mon Jul 18 13:49:08 2016

This script generates all json files required to run obs4MIPs data creation and
then calls cmor3 to generate two example files.

The file is split into 3 sections:
    1 - creates controlled vocabulary (CV) json inputs for CMOR3
    2 - creates an Omon (ocean monthly) json input for generating a 'tos'
        variable from a CF-compliant input
    3 - creates an fx (fixed field) json input for generating an 'areacello'
        variable from a CF-compliant input

PJD 20 Jul 2016     - Removed source_id
PJD 20 Jul 2016     - Further tweaks to enhance readability
                    - TODO:

@author: durack1
"""

#%% Import statements
import cmor,gc,json,os,ssl,sys,urllib2
import cdms2 as cdm
import numpy as np

#%% Set local path
homePath = os.path.join('/','/'.join(os.path.realpath(__file__).split('/')[0:-1])) ; # Extract path from executing file
#homePath = '/export/durack1/git/obs4MIPs-cmor-tables/' ; # Hard code path
#homePath = '/sync/git/obs4MIPs-cmor-tables/demo' ; # Hard code path
os.chdir(homePath)

#%% Function definitions

# Loop through input tables
def readJsonCreateDict(buildList):
    '''
    Documentation for readJsonCreateDict(buildList):
    -------
    The readJsonCreateDict() function reads web-based json files and writes
    their contents to a dictionary in memory

    Author: Paul J. Durack : pauldurack@llnl.gov

    Usage:
    ------
        >>> from runCMOR3 import readJsonCreateDict
        >>> readJsonCreateDict(['Omon','https://raw.githubusercontent.com/PCMDI/obs4MIPs-cmor-tables/master/Tables/obs4MIPs_Omon.json'])

    Notes:
    -----
        ...
    '''
    # Test for list input of length == 2
    if len(buildList[0]) != 2:
        print 'Invalid inputs, exiting..'
        sys.exit()
    # Create urllib2 context to deal with lab/LLNL web certificates
    ctx                 = ssl.create_default_context()
    ctx.check_hostname  = False
    ctx.verify_mode     = ssl.CERT_NONE
    # Iterate through buildList and write results to jsonDict
    jsonDict = {}       
    for count,table in enumerate(buildList):
        print 'Processing:',table[0]
        # Read web file
        jsonOutput = urllib2.urlopen(table[1], context=ctx)
        tmp = jsonOutput.read()
        vars()[table[0]] = tmp
        jsonOutput.close()
        # Write local json
        tmpFile = open('tmp.json','w')
        tmpFile.write(eval(table[0]))
        tmpFile.close()
        # Read local json
        vars()[table[0]] = json.load(open('tmp.json','r'))
        os.remove('tmp.json')
        jsonDict[table[0]] = eval(table[0]) ; # Write to dictionary
    
    return jsonDict

#%% SECTION 1 - Integrate all controlled vocabularies (CVs) into master file - create obs4MIPs_CV.json
jsonCVs = 'obs4MIPs_CV.json'
buildList = [
 ['frequency','https://raw.githubusercontent.com/PCMDI/obs4MIPs-cmor-tables/master/obs4MIPs_frequency.json'],
 ['grid_label','https://raw.githubusercontent.com/PCMDI/obs4MIPs-cmor-tables/master/obs4MIPs_grid_label.json'],
 ['grid_resolution','https://raw.githubusercontent.com/PCMDI/obs4MIPs-cmor-tables/master/obs4MIPs_grid_resolution.json'],
 ['grid','https://raw.githubusercontent.com/PCMDI/obs4MIPs-cmor-tables/master/obs4MIPs_grid.json'],
 ['institution_id','https://raw.githubusercontent.com/PCMDI/obs4MIPs-cmor-tables/master/obs4MIPs_institution_id.json'],
 ['mip_era','https://raw.githubusercontent.com/PCMDI/obs4MIPs-cmor-tables/master/obs4MIPs_mip_era.json'],
 ['product','https://raw.githubusercontent.com/PCMDI/obs4MIPs-cmor-tables/master/obs4MIPs_product.json'],
 ['realm','https://raw.githubusercontent.com/PCMDI/obs4MIPs-cmor-tables/master/obs4MIPs_realm.json'],
 ['required_global_attributes','https://raw.githubusercontent.com/PCMDI/obs4MIPs-cmor-tables/master/obs4MIPs_required_global_attributes.json'],
 ['table_id','https://raw.githubusercontent.com/PCMDI/obs4MIPs-cmor-tables/master/obs4MIPs_table_id.json'],
 ] ;

# Loop through buildList and create output tables
tmp = readJsonCreateDict(buildList)
for count,table in enumerate(tmp.keys()):
	vars()[table] = tmp[table].get(table)
#	if table == 'coordinate':
#		vars()[table] = tmp[table].get(table)
#	else:
#		vars()[table] = tmp[table]
del(tmp,count,table) ; gc.collect()

# Rebuild grid_labels
grid_labels = {}
grid_labels['gs1x1'] = { "grid_resolution":"1x1 degree" }
grid_labels['gs1x1 gn'] = { "grid_resolution":"1x1 degree" }
grid_labels['gs1x1 gr'] = { "grid_resolution":"1x1 degree" }
for count,grid in enumerate(grid_label):
    if count < 2:
        grid_labels[grid] = grid_resolution
    else:
        grid_labels[grid] = {}

# Rebuild
obs4MIPs_CV = {}
obs4MIPs_CV['CV'] = {}
obs4MIPs_CV['CV']['grid_labels'] = grid_labels
for count,CV in enumerate(buildList):
    CVName1 = CV[0]
    if CVName1 in ['grid','grid_label','grid_resolution']:
        continue ; # Exclude
    if CVName1 == 'coordinate':
        CVName2 = CVName1
        CVName1 = 'axis_entry'
    elif CVName1 == 'institution_id':
        CVName2 = CVName1
        CVName1 = 'institution_ids'
    elif CVName1 == 'source_id':
        CVName2 = CVName1
        CVName1 = 'source_ids'
    else:
        CVName2 = CVName1
    obs4MIPs_CV['CV'][CVName1] = eval(CVName2)

try:
  os.mkdir('obs4MIPs_CMOR_tables')
except:
  pass

outFile = 'obs4MIPs_CMOR_tables/obs4MIPs_CV.json'
# Check file exists
if os.path.exists(outFile):
    print 'File existing, purging:',outFile
    os.remove(outFile)
fH = open(outFile,'w')
json.dump(obs4MIPs_CV,fH,ensure_ascii=True,sort_keys=True,indent=4,separators=(',',':'),encoding="utf-8")
fH.close()



#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Mon Jul 18 13:49:08 2016

This script generates all json files required to run obs4MIPs data creation and
then calls cmor3 to generate two example files.

The file is split into 3 sections:
    1 - creates controlled vocabulary (CV) json inputs for CMOR3
    2 - creates an Omon (ocean monthly) json input for generating a 'tos'
        variable from a CF-compliant input
    3 - creates an fx (fixed field) json input for generating an 'areacello'
        variable from a CF-compliant input

PJD 20 Jul 2016     - Removed source_id
PJD 20 Jul 2016     - Further tweaks to enhance readability
PJD 28 Sep 2016     - Update to deal with new json (embedded key) formats
                    - TODO:

@author: durack1
"""

#%% Import statements
import cmor,gc,json,os,ssl,sys,urllib2
import cdms2 as cdm
import numpy as np

#%% Set local path
homePath = os.path.join('/','/'.join(os.path.realpath(__file__).split('/')[0:-1])) ; # Extract path from executing file
#homePath = '/export/durack1/git/obs4MIPs-cmor-tables/' ; # Hard code path
#homePath = '/sync/git/obs4MIPs-cmor-tables/demo' ; # Hard code path
os.chdir(homePath)

#%% Function definitions

# Loop through input tables
def readJsonCreateDict(buildList):
    '''
    Documentation for readJsonCreateDict(buildList):
    -------
    The readJsonCreateDict() function reads web-based json files and writes
    their contents to a dictionary in memory

    Author: Paul J. Durack : pauldurack@llnl.gov

    Usage:
    ------
        >>> from runCMOR3 import readJsonCreateDict
        >>> readJsonCreateDict(['Omon','https://raw.githubusercontent.com/PCMDI/obs4MIPs-cmor-tables/master/Tables/obs4MIPs_Omon.json'])

    Notes:
    -----
        ...
    '''
    # Test for list input of length == 2
    if len(buildList[0]) != 2:
        print 'Invalid inputs, exiting..'
        sys.exit()
    # Create urllib2 context to deal with lab/LLNL web certificates
    ctx                 = ssl.create_default_context()
    ctx.check_hostname  = False
    ctx.verify_mode     = ssl.CERT_NONE
    # Iterate through buildList and write results to jsonDict
    jsonDict = {}
    for count,table in enumerate(buildList):
        print 'Processing:',table[0]
        # Read web file
        jsonOutput = urllib2.urlopen(table[1], context=ctx)
        tmp = jsonOutput.read()
        vars()[table[0]] = tmp
        jsonOutput.close()
        # Write local json
        tmpFile = open('tmp.json','w')
        tmpFile.write(eval(table[0]))
        tmpFile.close()
        # Read local json
        vars()[table[0]] = json.load(open('tmp.json','r'))
        os.remove('tmp.json')
        jsonDict[table[0]] = eval(table[0]) ; # Write to dictionary

    return jsonDict

#%% SECTION 1 - Integrate all controlled vocabularies (CVs) into master file - create obs4MIPs_CV.json
jsonCVs = 'obs4MIPs_CV.json'
buildList = [
 ['frequency','https://raw.githubusercontent.com/PCMDI/obs4MIPs-cmor-tables/master/obs4MIPs_frequency.json'],
 ['grid_label','https://raw.githubusercontent.com/PCMDI/obs4MIPs-cmor-tables/master/obs4MIPs_grid_label.json'],
 ['grid_resolution','https://raw.githubusercontent.com/PCMDI/obs4MIPs-cmor-tables/master/obs4MIPs_grid_resolution.json'],
 ['grid','https://raw.githubusercontent.com/PCMDI/obs4MIPs-cmor-tables/master/obs4MIPs_grid.json'],
 ['institution_id','https://raw.githubusercontent.com/PCMDI/obs4MIPs-cmor-tables/master/obs4MIPs_institution_id.json'],
 ['mip_era','https://raw.githubusercontent.com/PCMDI/obs4MIPs-cmor-tables/master/obs4MIPs_mip_era.json'],
 ['product','https://raw.githubusercontent.com/PCMDI/obs4MIPs-cmor-tables/master/obs4MIPs_product.json'],
 ['realm','https://raw.githubusercontent.com/PCMDI/obs4MIPs-cmor-tables/master/obs4MIPs_realm.json'],
 ['required_global_attributes','https://raw.githubusercontent.com/PCMDI/obs4MIPs-cmor-tables/master/obs4MIPs_required_global_attributes.json'],
 ['table_id','https://raw.githubusercontent.com/PCMDI/obs4MIPs-cmor-tables/master/obs4MIPs_table_id.json'],
 ] ;

# Loop through buildList and create output tables
tmp = readJsonCreateDict(buildList)
for count,table in enumerate(tmp.keys()):
    vars()[table] = tmp[table]
del(tmp,count,table) ; gc.collect()

# Rebuild grid_labels
grid_labels = {}
grid_labels['gs1x1'] = { "grid_resolution":"1x1 degree" }
grid_labels['gs1x1 gn'] = { "grid_resolution":"1x1 degree" }
grid_labels['gs1x1 gr'] = { "grid_resolution":"1x1 degree" }
for count,grid in enumerate(grid_label):
    if count < 2:
        grid_labels[grid] = grid_resolution
    else:
        grid_labels[grid] = {}

# Rebuild
obs4MIPs_CV = {}
obs4MIPs_CV['CV'] = {}
obs4MIPs_CV['CV']['grid_labels'] = grid_labels
for count,CV in enumerate(buildList):
    CVName1 = CV[0]
    if CVName1 in ['grid','grid_label','grid_resolution']:
        continue ; # Exclude
    if CVName1 == 'coordinate':
        CVName2 = CVName1
        CVName1 = 'axis_entry'
    elif CVName1 == 'institution_id':
        CVName2 = CVName1
        CVName1 = 'institution_ids'
    elif CVName1 == 'source_id':
        CVName2 = CVName1
        CVName1 = 'source_ids'
    else:
        CVName2 = CVName1
    obs4MIPs_CV['CV'][CVName1] = eval(CVName2)

outFile = 'obs4MIPs_CV.json'
# Check file exists
if os.path.exists(outFile):
    print 'File existing, purging:',outFile
    os.remove(outFile)
fH = open(outFile,'w')
json.dump(obs4MIPs_CV,fH,ensure_ascii=True,sort_keys=True,indent=4,separators=(',',':'),encoding="utf-8")
fH.close()


#%% SECTION 2 - Integrate Omon into master file - create obs4MIPs_Omon_composite.json
jsonOmon = 'obs4MIPs_Omon_composite.json'
buildList = [
 ['coordinate','https://raw.githubusercontent.com/PCMDI/obs4MIPs-cmor-tables/master/obs4MIPs_coordinate.json'],
 ['Omon','https://raw.githubusercontent.com/PCMDI/obs4MIPs-cmor-tables/master/Tables/obs4MIPs_Omon.json']
 ] ;

# Loop through buildList and create output tables
tmp = readJsonCreateDict(buildList)
for count,table in enumerate(tmp.keys()):
    if table == 'coordinate':
        vars()[table] = tmp[table].get(table)
    else:
        vars()[table] = tmp[table]
del(tmp,count,table) ; gc.collect()

# Rebuild
table = {}
for count,CV in enumerate(buildList):
    CVName1 = CV[0]
    if CVName1 == 'coordinate':
        table['axis_entry'] = eval(CVName1)
    else:
        keys = eval(CVName1).keys()
        for count in range(len(keys)):
            table[keys[count]] = eval(CVName1).get(keys[count])

outFile = jsonOmon
# Check file exists
if os.path.exists(outFile):
    print 'File existing, purging:',outFile
    os.remove(outFile)
fH = open(outFile,'w')
json.dump(table,fH,ensure_ascii=True,sort_keys=True,indent=4,separators=(',',':'),encoding="utf-8")
fH.close()

# Process variable (with time axis)
cmor.setup(inpath='../Tables',netcdf_file_action=cmor.CMOR_REPLACE_4)
cmor.dataset_json('drive_obs4MIPs.json') ; # Update contents of this file to set your global_attributes
f       = cdm.open('amipobs_tos_360x180_v1.1.0_187001-187112.nc')
d       = f['tos']
lat     = d.getLatitude()
lon     = d.getLongitude()
time    = d.getTime()
#cmor.set_cur_dataset_attribute('history',f.history) ; # Force local file attribute as history
table   = 'obs4MIPs_Omon_composite.json' ; # Amon,Lmon,Omon,SImon
obs4MIPsOmonID = cmor.load_table(table) ; # Load target table (above), axis info (coordinates, grid*) and CVs
axes    = [ {'table_entry': 'time',
             'units': 'days since 1870-01-01',
             },
             {'table_entry': 'latitude',
              'units': 'degrees_north',
              'coord_vals': lat[:],
              'cell_bounds': lat.getBounds()},
             {'table_entry': 'longitude',
              'units': 'degrees_east',
              'coord_vals': lon[:],
              'cell_bounds': lon.getBounds()},
          ]
axis_ids = list()
for axis in axes:
    axis_id = cmor.axis(**axis)
    axis_ids.append(axis_id)
varid   = cmor.variable('tos',d.units,axis_ids)
values  = np.array(d[:],np.float32)
cmor.set_deflate(varid,1,1,1) ; # shuffle=1,deflate=1,deflate_level=1 ; CMOR 3.0.6+
cmor.write(varid,values,time_vals=time[:],time_bnds=time.getBounds()) ; # Write variable with time axis
f.close()
cmor.close()
# Cleanup
del(f,d,lat,lon,time) ; gc.collect()


#%% SECTION 3 - Integrate fx into master file - create obs4MIPs_fx_composite.json
jsonFx = 'obs4MIPs_fx_composite.json'
buildList = [
 ['coordinate','https://raw.githubusercontent.com/PCMDI/obs4MIPs-cmor-tables/master/obs4MIPs_coordinate.json'],
 ['fx','https://raw.githubusercontent.com/PCMDI/obs4MIPs-cmor-tables/master/Tables/obs4MIPs_fx.json']
 ] ;

# Loop through buildList and create output tables
tmp = readJsonCreateDict(buildList)
for count,table in enumerate(tmp.keys()):
    if table == 'coordinate':
        vars()[table] = tmp[table].get(table)
    else:
        vars()[table] = tmp[table]
del(tmp,count,table) ; gc.collect()

# Rebuild
table = {}
for count,CV in enumerate(buildList):
    CVName1 = CV[0]
    if CVName1 == 'coordinate':
        table['axis_entry'] = eval(CVName1)
    else:
        keys = eval(CVName1).keys()
        for count in range(len(keys)):
            table[keys[count]] = eval(CVName1).get(keys[count])

table['Header']['realm'] = 'ocean' ; # Overwrite realm info

outFile = jsonFx
# Check file exists
if os.path.exists(outFile):
    print 'File existing, purging:',outFile
    os.remove(outFile)
fH = open(outFile,'w')
json.dump(table,fH,ensure_ascii=True,sort_keys=True,indent=4,separators=(',',':'),encoding="utf-8")
fH.close()

# Process fixed field
cmor.setup(inpath='../Tables',netcdf_file_action=cmor.CMOR_REPLACE_4)
cmor.dataset_json('drive_obs4MIPs.json') ; # Update contents of this file to set your global_attributes
f       = cdm.open('amipbc_areacello_360x180_v1.1.0.nc')
d       = f['areacello']
lat     = d.getLatitude()
lon     = d.getLongitude()
time    = d.getTime()
#cmor.set_cur_dataset_attribute('history',f.history) ; # Force local file attribute as history
table   = 'obs4MIPs_fx_composite.json' ; # Amon,Lmon,Omon,SImon
cmor.load_table(table)
axes    = [  {'table_entry': 'latitude',
              'units': 'degrees_north',
              'coord_vals': lat[:],
              'cell_bounds': lat.getBounds()},
             {'table_entry': 'longitude',
              'units': 'degrees_east',
              'coord_vals': lon[:],
              'cell_bounds': lon.getBounds()},
          ]
axis_ids = list()
for axis in axes:
    axis_id = cmor.axis(**axis)
    axis_ids.append(axis_id)
varid   = cmor.variable('areacello',d.units,axis_ids)
values  = np.array(d[:],np.float32)
cmor.set_deflate(varid,1,1,1) ; # shuffle=1,deflate=1,deflate_level=1 ; CMOR 3.0.6+
cmor.write(varid,values) ; # Write fixed variable
f.close()
cmor.close()
# Cleanup
del(outFile,f,d,lat,lon,time) ; gc.collect()



# -*- coding: utf-8 -*-
"""
Created on Mon Jul 18 14:32:30 2016

@author: durack1
"""

import cdms2 as cdm

# Set nc classic as outputs
cdm.setCompressionWarnings(0) ; # Suppress warnings
cdm.setNetcdfShuffleFlag(0)
cdm.setNetcdfDeflateFlag(1) ; # was 0 130717
cdm.setNetcdfDeflateLevelFlag(9) ; # was 0 130717
cdm.setAutoBounds(1) ; # Ensure bounds on time and depth axes are generated

#%% Set input file and extract
f = '/work/durack1/Shared/150219_AMIPForcingData/360x180_v1.1.0_san/amipobs_tos_360x180_v1.1.0_187001-201512.nc'
fH = cdm.open(f)
d = fH('tos',time=('1870','1872'))
fo = '/export/durack1/git/obs4MIPs-cmor-tables/demo/amipobs_tos_360x180_v1.1.0_187001-187112.nc'
foH = cdm.open(fo,'w')
foH.write(d)
foH.close()
fH.close()